﻿
Write-Output "Stating!`n"

# Parameters
$SiteCode = "CT1"
$ProviderMachineName = "CTSCCM01.coretech.intra"

$ConfigMgrBoxPath = "C:\Program Files\Microsoft Configuration Manager\inboxes\auth\dataldr.box\BADMIFS"

$ModuleName = ".\ConfigurationManager.psd1"
$ModulePath = "C:\Program Files (x86)\Microsoft Configuration Manager\AdminConsole\bin"

$DevCollection = 'CH Inventory Missing'

Try{

    Function Import-CMLibraryModule{

        Set-Location -Path $ModulePath

        # This requires: https://www.microsoft.com/en-us/download/confirmation.aspx?id=46681
        Write-Output "Importing module ""$ModuleName"" from path: $ModulePath`n"
        Import-Module $ModuleName -ErrorAction Stop
    }

    Function Connect-CMSite{

        # Connect to the site's drive if it is not already present
        if((Get-PSDrive -Name $SiteCode -PSProvider CMSite -ErrorAction SilentlyContinue) -eq $null){

            New-PSDrive -Name $SiteCode -PSProvider CMSite -Root $ProviderMachineName
        }

        # Set the current location to be the site code

        Write-Output "Changing location to the site: $SiteCode, $ProviderMachineName`n"
        Set-Location "$($SiteCode):\" -Verbose
    }

    # Script Entry Point

    Import-CMLibraryModule

    Connect-CMSite

    Write-Output "Getting PC names from broken MIF files`n"
    $PCs = Get-ChildItem -Path $ConfigMgrBoxPath -Include *.MIF -Recurse -Force -ErrorAction SilentlyContinue | 
        ForEach-Object{ 

            $File = $_.FullName

            Try{

                (Get-Content -ReadCount 1 -TotalCount 6 -Path $_.FullName -ErrorAction Stop  | 
                    Select-String -Pattern "//KeyAttribute<NetBIOS\sName><(?<ComputerName>.*)>" -ErrorAction Stop).Matches.Groups[-1].Value
            }
            Catch{

                Write-Warning -Message "Failed for $File"
            }
        }

    # Get only unique names
    $Pcs = $Pcs | Select-Object -Unique
    Write-Output "Got the following unique names: $PCs`n"

    # Check if the device collection exists, create it if it doesnt
    if((Get-CMDeviceCollection -Name $DevCollection) -eq $null){

        Write-Output "Did not find an exisiting device collection with the name: ""$DevCollection"". Creating!`n"
        New-CMDeviceCollection -LimitingCollectionName "All Workstations" -Name $DevCollection | Out-Null
    }
    else{

        Write-Output "Device Collection: ""$DevCollection"" already exists`n"
    }

    # Remove everything in the collection
    if((Get-CMDeviceCollectionDirectMembershipRule -CollectionName $DevCollection) -ne $null){
   
       Write-Output "Removing computers from the collection"
       Get-CMDeviceCollectionDirectMembershipRule -CollectionName $DevCollection |
            ForEach-Object {

                Write-Output "`nRemoving PC: $($_.RuleName)"
                Remove-CMDeviceCollectionDirectMembershipRule -CollectionName $DevCollection -ResourceId $_.ResourceID -Force -Verbose
            }
    }

    # Add new members to the collection
    Foreach($item in $PCs){

        Write-Output "`nTrying to get: $item from CMDevices"
        $CMDevice = Get-CMDevice -Name $item -ErrorAction SilentlyContinue
    
        if($CMDevice -ne $null){

            Write-Output "Got: $($CMDevice.Name), with the ResourceID of $($CMDevice.ResourceID)"
        }
        else{

            Write-Output "Got nothing!"
        }

        if((Get-CMCollectionDirectMembershipRule -CollectionName $DevCollection -ResourceName $CMDevice.Name) -eq $null){

            Write-Output "Device doesnt already exist in the collection. Adding ""$item"" to ""$DevCollection"""
            Add-CMDeviceCollectionDirectMembershipRule -CollectionName $DevCollection -ResourceId $CMDevice.ResourceID -Verbose -ErrorAction Continue
        }
    }

    Write-Output "Exiting!"
}
Catch{

    Throw $Error[0]
}